@@ -1,11 +1,12 @@
 // std::moneypunct implementation details, GNU version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -13,19 +14,14 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 
-// You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-// USA.
+// Under Section 7 of GPL version 3, you are granted additional
+// permissions described in the GCC Runtime Library Exception, version
+// 3.1, as published by the Free Software Foundation.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// You should have received a copy of the GNU General Public License and
+// a copy of the GCC Runtime Library Exception along with this program;
+// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
+// <http://www.gnu.org/licenses/>.
 
 //
 // ISO C++ 14882: 22.2.6.3.2  moneypunct virtual functions
@@ -36,8 +32,8 @@
 #include <locale>
 #include <bits/c++locale_internal.h>
 
-namespace std
-{
+_GLIBCXX_BEGIN_NAMESPACE(std)
+
   // Construct and return valid pattern consisting of some combination of:
   // space none symbol sign value
   money_base::pattern
@@ -219,6 +215,7 @@ namespace std
 	  _M_data->_M_thousands_sep = ',';
 	  _M_data->_M_grouping = "";
 	  _M_data->_M_grouping_size = 0;
+	  _M_data->_M_use_grouping = false;
 	  _M_data->_M_curr_symbol = "";
 	  _M_data->_M_curr_symbol_size = 0;
 	  _M_data->_M_positive_sign = "";
@@ -239,11 +236,35 @@ namespace std
 							__cloc));
 	  _M_data->_M_thousands_sep = *(__nl_langinfo_l(__MON_THOUSANDS_SEP, 
 							__cloc));
-	  _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
-	  _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
 	  _M_data->_M_positive_sign = __nl_langinfo_l(__POSITIVE_SIGN, __cloc);
 	  _M_data->_M_positive_sign_size = strlen(_M_data->_M_positive_sign);
 
+	  // Check for NULL, which implies no grouping.
+	  if (_M_data->_M_thousands_sep == '\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_grouping = "";
+	      _M_data->_M_grouping_size = 0;
+	      _M_data->_M_use_grouping = false;
+	      _M_data->_M_thousands_sep = ',';
+	    }
+	  else
+	    {
+	      _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
+	      _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+	    }
+
+	  // Check for NULL, which implies no fractional digits.
+	  if (_M_data->_M_decimal_point == '\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_frac_digits = 0;
+	      _M_data->_M_decimal_point = '.';
+	    }
+	  else
+	    _M_data->_M_frac_digits = *(__nl_langinfo_l(__INT_FRAC_DIGITS, 
+							__cloc));
+
 	  char __nposn = *(__nl_langinfo_l(__INT_N_SIGN_POSN, __cloc));
 	  if (!__nposn)
 	    _M_data->_M_negative_sign = "()";
@@ -255,8 +276,6 @@ namespace std
 	  // _Intl == true
 	  _M_data->_M_curr_symbol = __nl_langinfo_l(__INT_CURR_SYMBOL, __cloc);
 	  _M_data->_M_curr_symbol_size = strlen(_M_data->_M_curr_symbol);
-	  _M_data->_M_frac_digits = *(__nl_langinfo_l(__INT_FRAC_DIGITS, 
-						      __cloc));
 	  char __pprecedes = *(__nl_langinfo_l(__INT_P_CS_PRECEDES, __cloc));
 	  char __pspace = *(__nl_langinfo_l(__INT_P_SEP_BY_SPACE, __cloc));
 	  char __pposn = *(__nl_langinfo_l(__INT_P_SIGN_POSN, __cloc));
@@ -284,6 +303,7 @@ namespace std
 	  _M_data->_M_thousands_sep = ',';
 	  _M_data->_M_grouping = "";
 	  _M_data->_M_grouping_size = 0;
+	  _M_data->_M_use_grouping = false;
 	  _M_data->_M_curr_symbol = "";
 	  _M_data->_M_curr_symbol_size = 0;
 	  _M_data->_M_positive_sign = "";
@@ -304,11 +324,35 @@ namespace std
 							__cloc));
 	  _M_data->_M_thousands_sep = *(__nl_langinfo_l(__MON_THOUSANDS_SEP, 
 							__cloc));
-	  _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
-	  _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
 	  _M_data->_M_positive_sign = __nl_langinfo_l(__POSITIVE_SIGN, __cloc);
 	  _M_data->_M_positive_sign_size = strlen(_M_data->_M_positive_sign);
 
+	  // Check for NULL, which implies no grouping.
+	  if (_M_data->_M_thousands_sep == '\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_grouping = "";
+	      _M_data->_M_grouping_size = 0;
+	      _M_data->_M_use_grouping = false;
+	      _M_data->_M_thousands_sep = ',';
+	    }
+	  else
+	    {
+	      _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
+	      _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+	    }
+
+	  // Check for NULL, which implies no fractional digits.
+	  if (_M_data->_M_decimal_point == '\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_frac_digits = 0;
+	      _M_data->_M_decimal_point = '.';
+	    }
+	  else
+	    _M_data->_M_frac_digits = *(__nl_langinfo_l(__FRAC_DIGITS,
+							__cloc));
+
 	  char __nposn = *(__nl_langinfo_l(__N_SIGN_POSN, __cloc));
 	  if (!__nposn)
 	    _M_data->_M_negative_sign = "()";
@@ -320,7 +364,6 @@ namespace std
 	  // _Intl == false
 	  _M_data->_M_curr_symbol = __nl_langinfo_l(__CURRENCY_SYMBOL, __cloc);
 	  _M_data->_M_curr_symbol_size = strlen(_M_data->_M_curr_symbol);
-	  _M_data->_M_frac_digits = *(__nl_langinfo_l(__FRAC_DIGITS, __cloc));
 	  char __pprecedes = *(__nl_langinfo_l(__P_CS_PRECEDES, __cloc));
 	  char __pspace = *(__nl_langinfo_l(__P_SEP_BY_SPACE, __cloc));
 	  char __pposn = *(__nl_langinfo_l(__P_SIGN_POSN, __cloc));
@@ -361,6 +404,7 @@ namespace std
 	  _M_data->_M_thousands_sep = L',';
 	  _M_data->_M_grouping = "";
 	  _M_data->_M_grouping_size = 0;
+	  _M_data->_M_use_grouping = false;
 	  _M_data->_M_curr_symbol = L"";
 	  _M_data->_M_curr_symbol_size = 0;
 	  _M_data->_M_positive_sign = L"";
@@ -383,7 +427,10 @@ namespace std
 	  __c_locale __old = __uselocale(__cloc);
 #else
 	  // Switch to named locale so that mbsrtowcs will work.
-	  char* __old = strdup(setlocale(LC_ALL, NULL));
+	  char* __old = setlocale(LC_ALL, NULL);
+          const size_t __llen = strlen(__old) + 1;
+          char* __sav = new char[__llen];
+          memcpy(__sav, __old, __llen);
 	  setlocale(LC_ALL, __name);
 #endif
 
@@ -393,8 +440,32 @@ namespace std
 
 	  __u.__s = __nl_langinfo_l(_NL_MONETARY_THOUSANDS_SEP_WC, __cloc);
 	  _M_data->_M_thousands_sep = __u.__w;
-	  _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
-	  _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+
+	  // Check for NULL, which implies no grouping.
+	  if (_M_data->_M_thousands_sep == L'\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_grouping = "";
+	      _M_data->_M_grouping_size = 0;
+	      _M_data->_M_use_grouping = false;
+	      _M_data->_M_thousands_sep = L',';
+	    }
+	  else
+	    {
+	      _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
+	      _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+	    }
+
+	  // Check for NULL, which implies no fractional digits.
+	  if (_M_data->_M_decimal_point == L'\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_frac_digits = 0;
+	      _M_data->_M_decimal_point = L'.';
+	    }
+	  else
+	    _M_data->_M_frac_digits = *(__nl_langinfo_l(__INT_FRAC_DIGITS, 
+							__cloc));
 
 	  const char* __cpossign = __nl_langinfo_l(__POSITIVE_SIGN, __cloc);
 	  const char* __cnegsign = __nl_langinfo_l(__NEGATIVE_SIGN, __cloc);
@@ -403,7 +474,7 @@ namespace std
 	  wchar_t* __wcs_ps = 0;
 	  wchar_t* __wcs_ns = 0;
 	  const char __nposn = *(__nl_langinfo_l(__INT_N_SIGN_POSN, __cloc));
-	  try
+	  __try
 	    {
 	      mbstate_t __state;
 	      size_t __len = strlen(__cpossign);
@@ -448,23 +519,21 @@ namespace std
 		_M_data->_M_curr_symbol = L"";
 	      _M_data->_M_curr_symbol_size = wcslen(_M_data->_M_curr_symbol);
 	    }
-	  catch (...)
+	  __catch(...)
 	    {
 	      delete _M_data;
 	      _M_data = 0;
-	      delete __wcs_ps;
-	      delete __wcs_ns;	      
+	      delete [] __wcs_ps;
+	      delete [] __wcs_ns;	      
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2)
 	      __uselocale(__old);
 #else
-	      setlocale(LC_ALL, __old);
-	      free(__old);
+	      setlocale(LC_ALL, __sav);
+	      delete [] __sav;
 #endif
 	      __throw_exception_again;
 	    } 
 	  
-	  _M_data->_M_frac_digits = *(__nl_langinfo_l(__INT_FRAC_DIGITS, 
-						      __cloc));
 	  char __pprecedes = *(__nl_langinfo_l(__INT_P_CS_PRECEDES, __cloc));
 	  char __pspace = *(__nl_langinfo_l(__INT_P_SEP_BY_SPACE, __cloc));
 	  char __pposn = *(__nl_langinfo_l(__INT_P_SIGN_POSN, __cloc));
@@ -478,8 +547,8 @@ namespace std
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2)
 	  __uselocale(__old);
 #else
-	  setlocale(LC_ALL, __old);
-	  free(__old);
+	  setlocale(LC_ALL, __sav);
+	  delete [] __sav;
 #endif
 	}
     }
@@ -503,6 +572,7 @@ namespace std
 	  _M_data->_M_thousands_sep = L',';
 	  _M_data->_M_grouping = "";
           _M_data->_M_grouping_size = 0;
+	  _M_data->_M_use_grouping = false;
 	  _M_data->_M_curr_symbol = L"";
 	  _M_data->_M_curr_symbol_size = 0;
 	  _M_data->_M_positive_sign = L"";
@@ -525,7 +595,10 @@ namespace std
 	  __c_locale __old = __uselocale(__cloc);
 #else
 	  // Switch to named locale so that mbsrtowcs will work.
-	  char* __old = strdup(setlocale(LC_ALL, NULL));
+	  char* __old = setlocale(LC_ALL, NULL);
+          const size_t __llen = strlen(__old) + 1;
+          char* __sav = new char[__llen];
+          memcpy(__sav, __old, __llen);
 	  setlocale(LC_ALL, __name);
 #endif
 
@@ -535,8 +608,32 @@ namespace std
 
 	  __u.__s = __nl_langinfo_l(_NL_MONETARY_THOUSANDS_SEP_WC, __cloc);
 	  _M_data->_M_thousands_sep = __u.__w;
-	  _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
-          _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+
+	  // Check for NULL, which implies no grouping.
+	  if (_M_data->_M_thousands_sep == L'\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_grouping = "";
+	      _M_data->_M_grouping_size = 0;
+	      _M_data->_M_use_grouping = false;
+	      _M_data->_M_thousands_sep = L',';
+	    }
+	  else
+	    {
+	      _M_data->_M_grouping = __nl_langinfo_l(__MON_GROUPING, __cloc);
+	      _M_data->_M_grouping_size = strlen(_M_data->_M_grouping);
+	    }
+
+	  // Check for NULL, which implies no fractional digits.
+	  if (_M_data->_M_decimal_point == L'\0')
+	    {
+	      // Like in "C" locale.
+	      _M_data->_M_frac_digits = 0;
+	      _M_data->_M_decimal_point = L'.';
+	    }
+	  else
+	    _M_data->_M_frac_digits = *(__nl_langinfo_l(__FRAC_DIGITS,
+							__cloc));
 
 	  const char* __cpossign = __nl_langinfo_l(__POSITIVE_SIGN, __cloc);
 	  const char* __cnegsign = __nl_langinfo_l(__NEGATIVE_SIGN, __cloc);
@@ -545,7 +642,7 @@ namespace std
 	  wchar_t* __wcs_ps = 0;
 	  wchar_t* __wcs_ns = 0;
 	  const char __nposn = *(__nl_langinfo_l(__N_SIGN_POSN, __cloc));
-	  try
+	  __try
             {
               mbstate_t __state;
               size_t __len;
@@ -591,22 +688,21 @@ namespace std
 		_M_data->_M_curr_symbol = L"";
               _M_data->_M_curr_symbol_size = wcslen(_M_data->_M_curr_symbol);
 	    }
-          catch (...)
+          __catch(...)
 	    {
 	      delete _M_data;
               _M_data = 0;
-	      delete __wcs_ps;
-	      delete __wcs_ns;	      
+	      delete [] __wcs_ps;
+	      delete [] __wcs_ns;	      
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2)
 	      __uselocale(__old);
 #else
-	      setlocale(LC_ALL, __old);
-	      free(__old);
+	      setlocale(LC_ALL, __sav);
+	      delete [] __sav;
 #endif
               __throw_exception_again;
 	    }
 
-	  _M_data->_M_frac_digits = *(__nl_langinfo_l(__FRAC_DIGITS, __cloc));
 	  char __pprecedes = *(__nl_langinfo_l(__P_CS_PRECEDES, __cloc));
 	  char __pspace = *(__nl_langinfo_l(__P_SEP_BY_SPACE, __cloc));
 	  char __pposn = *(__nl_langinfo_l(__P_SIGN_POSN, __cloc));
@@ -620,8 +716,8 @@ namespace std
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2)
 	  __uselocale(__old);
 #else
-	  setlocale(LC_ALL, __old);
-	  free(__old);
+	  setlocale(LC_ALL, __sav);
+	  delete [] __sav;
 #endif
 	}
     }
@@ -652,4 +748,5 @@ namespace std
       delete _M_data;
     }
 #endif
-}
+
+_GLIBCXX_END_NAMESPACE
