@@ -1,11 +1,12 @@
 // { dg-require-namedlocale "" }
 
-// Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,9 +15,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 #include <sstream>
 #include <locale>
@@ -38,8 +38,12 @@ int main()
   long double ld = -1234567890;
   mp.put(oss.rdbuf(), true, oss, ' ', ld); // -EUR  12.345.678,90
   string res = oss.str();
-  
+
+#if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 7)
+  VERIFY( res == "-12.345.678,90 EUR " );
+#else
   VERIFY( res == "-EUR  12.345.678,90" );
+#endif
 
   return 0;
 }
